@@ -18,7 +18,7 @@
 # the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 
 import os, copy, time
-import preludedb, CheetahFilters
+import prelude, preludedb, CheetahFilters
 
 import prewikka.views
 from prewikka import view, Config, Log, Database, IDMEFDatabase, \
@@ -61,21 +61,6 @@ def init_dataset(dataset, config, request):
     else:
         dataset["prewikka.external_link_target"] = "_self"
 
-    qstring = request.getQueryString()
-    if qstring[0:2] == "/?":
-        qstring = qstring[2:]
-
-    dataset["prewikka.url.current"] = qstring
-
-    referer = request.getReferer()
-    idx = referer.rfind("/")
-    if idx != -1:
-        referer = referer[idx + 1:]
-        if referer[0:1] == "?":
-            referer = referer[1:]
-
-    dataset["prewikka.url.referer"] = referer
-
     dataset["arguments"] = []
     for name, value in request.arguments.items():
         if name in ("_login", "_password"):
@@ -84,7 +69,7 @@ def init_dataset(dataset, config, request):
         if name == "view" and value == "logout":
             continue
 
-        dataset["arguments"].append((name, value))
+        dataset["arguments"].append((name, utils.toUnicode(value)))
 
 
 def load_template(name, dataset):
@@ -121,6 +106,15 @@ def get_core_from_config(path, threaded=False):
 
 
 class Core:
+    def _checkVersion(self):
+        self._prelude_version_error = None
+
+        if not prelude.prelude_check_version(siteconfig.libprelude_required_version):
+            self._prelude_version_error = "Prewikka %s require libprelude %s or higher" % (siteconfig.version, siteconfig.libprelude_required_version)
+
+        elif not preludedb.preludedb_check_version(siteconfig.libpreludedb_required_version):
+            self._prelude_version_error = "Prewikka %s require libpreludedb %s or higher" % (siteconfig.version, siteconfig.libpreludedb_required_version)
+
     def __init__(self, config=None):
         class Env: pass
         self._env = Env()
@@ -137,6 +131,8 @@ class Core:
 
         preludedb.preludedb_init()
 
+        self._checkVersion()
+
         self._database_schema_error = None
         try:
             self._initDatabase()
@@ -276,7 +272,7 @@ class Core:
                 print key + ":"
                 self._printDataSet(value, level + 1)
             else:
-                print "%s: %s" % (key, value)
+                print "%s: %s" % (key, repr(value))
 
     def _checkPermissions(self, request, view, user):
         if user and view.has_key("permissions"):
@@ -305,8 +301,9 @@ class Core:
         return user
 
     def prepareError(self, e, request, user, login, view):
-        self._env.log.error("%s" % str(e), request, login)
-        error = Error.PrewikkaUserError("Prewikka internal error", str(e),
+        e = unicode(e)
+        self._env.log.error(e, request, login)
+        error = Error.PrewikkaUserError("Prewikka internal error", e,
                                         display_traceback=not self._env.config.general.has_key("disable_error_traceback"))
         self._setupDataSet(error.dataset, request, user, view=view)
         return error
@@ -315,7 +312,12 @@ class Core:
         login = None
         view = None
         user = None
+        encoding = self._env.config.general.getOptionValue("encoding", "utf8")
+
         try:
+            if self._prelude_version_error:
+                raise Error.PrewikkaUserError("Version Requirement error", self._prelude_version_error)
+
             if self._database_schema_error != None:
                 raise Error.PrewikkaUserError("Database error", self._database_schema_error)
 
@@ -340,7 +342,7 @@ class Core:
 
         except Error.PrewikkaUserError, e:
             if e._log_priority:
-                self._env.log.log(e._log_priority, "%s" % str(e), request=request, user=login or e._log_user)
+                self._env.log.log(e._log_priority, unicode(e), request=request, user=login or e._log_user)
 
             self._setupDataSet(e.dataset, request, user, view=view)
             dataset, template_name = e.dataset, e.template
@@ -358,10 +360,11 @@ class Core:
         resolve.process(self._env.dns_max_delay)
 
         try:
-                request.content = str(template)
+                request.content = template.respond()
         except Exception, e:
             error = self.prepareError(e, request, user, login, view)
-            request.content = str(load_template(error.template, error.dataset))
+            request.content = load_template(error.template, error.dataset).respond()
 
+        request.content = request.content.encode(encoding, "xmlcharrefreplace")
         request.sendResponse()
 
