@@ -22,7 +22,7 @@ import time, types
 from prelude import *
 from preludedb import *
 
-from prewikka.utils import escape_html_string, time_to_ymdhms
+from prewikka.utils import escape_html_string, time_to_ymdhms, toUnicode
 
 
 
@@ -36,7 +36,7 @@ def escape_value(value):
 class IDMEFTime(object):
     def __init__(self, res):
         self._res = res
-        
+
     def __del__(self):
         idmef_time_destroy(self._res)
 
@@ -45,13 +45,13 @@ class IDMEFTime(object):
 
     def __int__(self):
         return idmef_time_get_sec(self._res)
-                
+
     def __float__(self):
         return float(idmef_time_get_sec(self._res)) + float(idmef_time_get_usec(self._res)) / 10 ** 6
-        
+
     def toYMDHMS(self):
         return time_to_ymdhms(time.localtime(idmef_time_get_sec(self._res)))
-        
+
     def __getattribute__(self, name):
         if name is "sec":
             return idmef_time_get_sec(self._res)
@@ -91,21 +91,21 @@ def convert_idmef_value(value):
             IDMEF_VALUE_TYPE_DOUBLE:        idmef_value_get_double,
             IDMEF_VALUE_TYPE_STRING:        idmef_value_get_string,
             IDMEF_VALUE_TYPE_DATA:          idmef_value_get_data,
-	    IDMEF_VALUE_TYPE_CLASS:	    idmef_value_get_object,
+            IDMEF_VALUE_TYPE_CLASS:         idmef_value_get_object,
             IDMEF_VALUE_TYPE_ENUM:          get_enum,
             IDMEF_VALUE_TYPE_TIME:          get_time
             }[idmef_value_get_type(value)](value)
     except KeyError:
         return None
 
-    
-        
+
+
 class Message:
     def __init__(self, res, htmlsafe):
         self._res = res
         self._value_list = None
         self._htmlsafe = htmlsafe
-        
+
     def __del__(self):
         idmef_message_destroy(self._res)
 
@@ -115,7 +115,7 @@ class Message:
     def __iter__(self):
         if not self._value_list:
             raise TypeError, "iteration over a non-sequence"
-        
+
         self._list_iterator = 0
         return self
 
@@ -124,7 +124,7 @@ class Message:
             return idmef_value_get_count(self._value_list)
 
         return 1
-    
+
     def next(self):
         next = idmef_value_get_nth(self._value_list, self._list_iterate)
         if not next:
@@ -132,7 +132,7 @@ class Message:
 
         value = self._convert_value(next, self._root + "(%d)" % self._list_iterate)
         self._list_iterate += 1
-        
+
         return value
 
     def _convert_value(self, idmef_value, key):
@@ -143,33 +143,36 @@ class Message:
             value._value_list = idmef_value
             if self._value_list:
                 idmef_value_ref(idmef_value)
-                
+
         elif idmef_value_get_type(idmef_value) != IDMEF_VALUE_TYPE_CLASS:
             value = convert_idmef_value(idmef_value)
             if not self._value_list:
                 idmef_value_destroy(idmef_value)
-            
+
+            if idmef_value_get_type(idmef_value) == IDMEF_VALUE_TYPE_STRING:
+                value = toUnicode(value)
+
         else:
             if not self._value_list:
                 idmef_value_destroy(idmef_value)
-                
+
             value = Message(idmef_message_ref(self._res), self._htmlsafe)
             value._root = key
 
         return value
-    
+
     def _get_raw_value(self, key):
-        path = idmef_path_new_fast(key)
+        path = idmef_path_new_fast(key.encode("utf8"))
         idmef_value = idmef_path_get(path, self._res)
-        
+
         if idmef_value:
             ret = self._convert_value(idmef_value, key)
         else:
-            if idmef_path_is_list(path, -1):
+            if idmef_path_is_ambiguous(path):
                 ret = []
             else:
                 ret = None
-            
+
         idmef_path_destroy(path)
         return ret
 
@@ -181,29 +184,29 @@ class Message:
             return escape_value(self._get_raw_value(key))
         else:
             return self._get_raw_value(key)
-             
+
     def match(self, criteria):
         if type(criteria) is list:
             criteria = " && ".join(criteria)
 
-        criteria = idmef_criteria_new_from_string(criteria)
+        criteria = idmef_criteria_new_from_string(criteria.encode("utf8"))
         ret = idmef_criteria_match(criteria, self._res)
         idmef_criteria_destroy(criteria)
 
         return ret
 
-    def get(self, key, default=None, htmlsafe=None):     
+    def get(self, key, default=None, htmlsafe=None):
         if htmlsafe != None:
             htmlsafe_bkp = self._htmlsafe
             self._htmlsafe = htmlsafe
-            
-        val = self[key] 
+
+        val = self[key]
         if val == None:
                 val = default
-        
+
         if htmlsafe != None:
             self._htmlsafe = htmlsafe_bkp
-        
+
         return val
 
     def getAdditionalData(self, searched, many_values=False):
@@ -213,13 +216,13 @@ class Message:
             meaning = self["%s.additional_data(%d).meaning" % (self._root, i)]
             if meaning is None:
                 break
-            
+
             if meaning == searched:
                 value = self["%s.additional_data(%d).data" % (self._root, i)]
-                
+
                 if not many_values:
                     return value
-                
+
                 values.append(value)
 
             i += 1
@@ -252,35 +255,35 @@ class DbResult:
         self._rows = [ ]
         self._has_cache = False
         self._res, self._len = results
-        
+
     def __iter__(self):
         if self._has_cache:
             return iter(self._rows)
         else:
             return self
-        
+
     def __len__(self):
         return self._len
-       
+
     def __del__(self):
         if self._res:
             self._db_delete(self._res)
-        
+
     def __getitem__(self, key):
         if isinstance(key, types.SliceType):
             start, stop, step = key.start, key.stop, key.step
             index = start + stop
         else:
             index = key
-        
+
         if not self._has_cache:
             for r in self:
                 if len(self._rows) >= index:
                     break
-                
+
         return self._rows[key]
-        
-    def next(self):        
+
+    def next(self):
         if self._res == None:
             raise StopIteration
 
@@ -292,26 +295,26 @@ class DbResult:
             raise StopIteration
 
         row = self._db_convert_row(values)
-       
+
         self._rows.append(row)
         return row
-            
+
 
 class DbResultValues(DbResult):
     def __init__(self, selection, results):
         self._selection = selection
         DbResult.__init__(self, results)
-                
+
     def _db_get_next(self):
         return preludedb_result_values_get_next(self._res)
-        
+
     def _db_delete(self, result):
         if self._selection:
             preludedb_path_selection_destroy(self._selection)
-            
+
         if result:
             preludedb_result_values_destroy(result)
-            
+
     def _db_convert_row(self, values):
         row = []
         for value in values:
@@ -319,34 +322,35 @@ class DbResultValues(DbResult):
                row.append(None)
            else:
                row.append(convert_idmef_value(value))
-               idmef_value_destroy(value)      
-               
+               idmef_value_destroy(value)
+
         return row
-        
+
 class DbResultIdents(DbResult):
     def _db_get_next(self):
         return preludedb_result_idents_get_next(self._res)
-        
+
     def _db_delete(self, result):
         if result:
             preludedb_result_idents_destroy(result)
-                    
+
     def _db_convert_row(self, value):
         return value
 
 class IDMEFDatabase:
     _db_destroy = preludedb_destroy
     _db = None
-    
+
     def __init__(self, config):
         settings = preludedb_sql_settings_new()
         for param in "file", "host", "port", "name", "user", "pass":
-            if config.getOptionValue(param):
-                preludedb_sql_settings_set(settings, param, config.getOptionValue(param))
+            value = config.getOptionValue(param)
+            if value:
+                preludedb_sql_settings_set(settings, param, value.encode("utf8"))
 
-        sql = preludedb_sql_new(config.getOptionValue("type", "mysql"), settings)
+        sql = preludedb_sql_new(config.getOptionValue("type", "mysql").encode("utf8"), settings)
         if config.getOptionValue("log"):
-            preludedb_sql_enable_query_logging(sql, config.getOptionValue("log"))
+            preludedb_sql_enable_query_logging(sql, config.getOptionValue("log").encode("utf8"))
 
         cur = ver = None
         wanted_version = "0.9.12"
@@ -360,44 +364,44 @@ class IDMEFDatabase:
                 raise "libpreludedb %s or higher is required (%s found)." % (wanted_version, cur)
             else:
                 raise "libpreludedb %s or higher is required." % wanted_version
-        
+
         self._db = preludedb_new(sql, None)
 
     def __del__(self):
         if self._db:
             self._db_destroy(self._db)
-            
+
     def _getMessageIdents(self, get_message_idents, criteria, limit, offset, order_by):
+        if len(criteria) == 0:
+            criteria = None
+
         if type(criteria) is list:
-            if len(criteria) == 0:
-                criteria = None
-            else:
-                criteria = " && ".join(criteria)
-                
+            criteria = " && ".join(criteria)
+
         if criteria:
-            criteria = idmef_criteria_new_from_string(criteria)
+            criteria = idmef_criteria_new_from_string(criteria.encode("utf8"))
 
         idents = [ ]
-    
+
         if order_by == "time_asc":
             order_by = PRELUDEDB_RESULT_IDENTS_ORDER_BY_CREATE_TIME_ASC
         else:
             order_by = PRELUDEDB_RESULT_IDENTS_ORDER_BY_CREATE_TIME_DESC
-            
-        try:    
+
+        try:
             result = get_message_idents(self._db, criteria, limit, offset, order_by)
         except:
             self._freeDbParams(criteria=criteria)
             raise
-                    
+
         if criteria:
             idmef_criteria_destroy(criteria)
-        
+
         if not result:
-            return [ ]            
-        
+            return [ ]
+
         return DbResultIdents(result)
-        
+
     def getAlertIdents(self, criteria=None, limit=-1, offset=-1, order_by="time_desc"):
         return self._getMessageIdents(preludedb_get_alert_idents2, criteria, limit, offset, order_by)
 
@@ -406,17 +410,20 @@ class IDMEFDatabase:
 
     def _getLastMessageIdent(self, type, get_message_idents, analyzerid):
         criteria = None
-        if analyzerid != None:
-            criteria = "%s.analyzer(-1).analyzerid == '%s'" % (type, str(analyzerid))
+        if analyzerid is not False:
+            if analyzerid is None:
+                criteria = "! %s.analyzer(-1).analyzerid" % (type)
+            else:
+                criteria = "%s.analyzer(-1).analyzerid == '%s'" % (type, unicode(analyzerid))
 
         idents = get_message_idents(criteria, limit=1)
 
         return idents[0]
 
-    def getLastAlertIdent(self, analyzer=None):
+    def getLastAlertIdent(self, analyzer=False):
         return self._getLastMessageIdent("alert", self.getAlertIdents, analyzer)
 
-    def getLastHeartbeatIdent(self, analyzer=None):
+    def getLastHeartbeatIdent(self, analyzer=False):
         return self._getLastMessageIdent("heartbeat", self.getHeartbeatIdents, analyzer)
 
     def getAlert(self, ident, htmlsafe=False):
@@ -442,23 +449,23 @@ class IDMEFDatabase:
     def _freeDbParams(self, selection=None, criteria=None):
         if selection:
             preludedb_path_selection_destroy(selection)
-        
+
         if criteria:
             idmef_criteria_destroy(criteria)
-            
+
     def getValues(self, selection, criteria=None, distinct=0, limit=-1, offset=-1):
         if type(criteria) is list:
             if len(criteria) == 0:
                 criteria = None
             else:
                 criteria = " && ".join([ "(" + c + ")" for c in criteria ])
-                            
+
         if criteria:
-            criteria = idmef_criteria_new_from_string(criteria)
-            
+            criteria = idmef_criteria_new_from_string(criteria.encode("utf8"))
+
         my_selection = preludedb_path_selection_new()
         for selected in selection:
-            my_selected = preludedb_selected_path_new_string(selected)
+            my_selected = preludedb_selected_path_new_string(selected.encode("utf8"))
             preludedb_path_selection_add(my_selection, my_selected)
 
         try:
@@ -466,19 +473,19 @@ class IDMEFDatabase:
         except:
             self._freeDbParams(my_selection, criteria)
             raise
-            
+
         if criteria:
-            idmef_criteria_destroy(criteria)       
-        
+            idmef_criteria_destroy(criteria)
+
         if not result:
             preludedb_path_selection_destroy(my_selection)
             return [ ]
-            
+
         return DbResultValues(my_selection, result)
-        
+
     def _countMessages(self, root, criteria):
         return self.getValues(["count(%s.create_time)" % root], criteria)[0][0]
-        
+
     def countAlerts(self, criteria=None):
         return self._countMessages("alert", criteria)
 
@@ -511,40 +518,31 @@ class IDMEFDatabase:
                 index += 1
             analyzer_paths.append(path)
 
-        return analyzer_paths            
+        return analyzer_paths
 
     def getAnalyzer(self, analyzerid):
         ident = self.getLastHeartbeatIdent(analyzerid)
         heartbeat = self.getHeartbeat(ident)
 
-        index = 0
-        while True:
-            if not heartbeat["heartbeat.analyzer(%d).name" % (index + 1)]:
-                break
-            index += 1
+        path = []
+        analyzer = {}
+        analyzerd = { "path": path, "node_addresses": [], "node_name": None, "node_location": None }
 
-        analyzer = { }
-        analyzer["analyzerid"] = analyzerid
-        analyzer["name"] = heartbeat.get("heartbeat.analyzer(%d).name" % index)
-        analyzer["model"] = heartbeat.get("heartbeat.analyzer(%d).model" % index) 
-        analyzer["version"] = heartbeat.get("heartbeat.analyzer(%d).version" % index)
-        analyzer["class"] = heartbeat.get("heartbeat.analyzer(%d).class" % index)
-        analyzer["ostype"] = heartbeat.get("heartbeat.analyzer(%d).ostype" % index)
-        analyzer["osversion"] = heartbeat.get("heartbeat.analyzer(%d).osversion" % index)
-        analyzer["node_name"] = heartbeat.get("heartbeat.analyzer(%d).node.name" % index)
-        analyzer["node_location"] = heartbeat.get("heartbeat.analyzer(%d).node.location" % index)
-                
-        i = 0
-        analyzer["node_addresses"] = [ ]
-        while True:
-            address = heartbeat.get("heartbeat.analyzer(%d).node.address(%d).address" % (index, i))
-            if not address:
-                break
-            analyzer["node_addresses"].append(address)
-            i += 1
-        
-        analyzer["last_heartbeat_time"] = heartbeat.get("heartbeat.create_time")
-        analyzer["last_heartbeat_interval"] = heartbeat["heartbeat.heartbeat_interval"]
-        analyzer["last_heartbeat_status"] = heartbeat.getAdditionalData("Analyzer status")
-        
-        return analyzer
+        for a in heartbeat["analyzer"]:
+            path.append(a["analyzerid"])
+            analyzer = a
+
+        for column in "analyzerid", "name", "model", "version", "class", "ostype", "osversion":
+            analyzerd[column] = analyzer.get(column, None)
+
+        analyzerd["node_name"] = analyzer.get("node.name")
+        analyzerd["node_location"] = analyzer.get("node.location")
+
+        for addr in analyzer.get("node.address.address", []):
+            analyzerd["node_addresses"].append(addr)
+
+        analyzerd["last_heartbeat_time"] = heartbeat.get("heartbeat.create_time")
+        analyzerd["last_heartbeat_interval"] = heartbeat.get("heartbeat.heartbeat_interval")
+        analyzerd["last_heartbeat_status"] = heartbeat.getAdditionalData("Analyzer status")
+
+        return analyzerd
